/* Hobbit's efforts at a unified s/key. */

/* Sleazed from code originally by Wietse Venema <wietse@wzv.win.tue.nl> */

#include <sys/types.h>
#include <sys/time.h>
#include <utmpx.h>
#include <string.h>

/* utmpx_swap - pony up a "logout" followed by a "login" */

utmpx_swap (line, user)
char   *line;
char   *user;
{
    struct utmpx *ut;
    struct utmpx clrutx;
    int     ret = (-1);

    /*
     * SYSV4 ttymon and login use tty port names with the "/dev/" prefix
     * stripped off. Rlogind and telnetd, on the other hand, make utmpx
     * entries with device names like /dev/pts/nnn. We therefore cannot use
     * getutxline(). Return nonzero if no utmp entry was found with our own
     * process ID for a login or user process.
     */

/* the above should be academic if we're past the first login, and at this
   point the PID is wrong anyways.  Whether or not the HOST is filled in
   is quite a different story... */

    strcpy (clrutx.ut_line, line);
    if ((ut = getutxline(&clrutx)) != 0) {
        if (ut->ut_type == LOGIN_PROCESS || ut->ut_type == USER_PROCESS) {

/* first, log out the pseudo user */
	    memcpy (&clrutx, ut, sizeof (clrutx));
	    clrutx.ut_type = DEAD_PROCESS;
	    clrutx.ut_exit.e_termination = 0;
	    clrutx.ut_exit.e_exit = 0;
	    gettimeofday (&(clrutx.ut_tv));
	    updwtmpx(WTMPX_FILE, &clrutx);

/* now swap in the real user, line, and update both */
	    strncpy(ut->ut_line, line, sizeof(ut->ut_line));
	    strncpy(ut->ut_user, user, sizeof(ut->ut_user));
	    gettimeofday(&(ut->ut_tv));
	    pututxline(ut);
	    updwtmpx(WTMPX_FILE, ut);
	    ret = 0;
	} /* proc type */
    } /* getutxline */
    endutxent();
    return (ret);
} /* utmpx_swap */
